<?php

namespace Drupal\usage_data\Plugin\views\field;

use Drupal\usage_data\UsageDataDatabaseStorage;
use Drupal\views\Plugin\views\field\FieldPluginBase;
use Drupal\views\ResultRow;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * A handler to show Usage Data counter fields for different event types.
 *
 * @ingroup views_field_handlers
 *
 * @ViewsField("usage_data_field")
 */
class UsageDataField extends FieldPluginBase {

  /**
   * Usage Data database storage.
   *
   * @var \Drupal\usage_data\UsageDataDatabaseStorage
   */
  protected $usageDataDatabaseStorage;

  /**
   * Plugin constructor.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\usage_data\UsageDataDatabaseStorage $usageDataDatabaseStorage
   *   Usage Data database storage.
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, UsageDataDatabaseStorage $usageDataDatabaseStorage) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->usageDataDatabaseStorage = $usageDataDatabaseStorage;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('usage_data.storage.database')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function query() {
    // Do nothing. Sit back, relax, have a mojito.
  }

  /**
   * {@inheritdoc}
   */
  public function getValue(ResultRow $values, $field = NULL) {
    if (empty($entity = $values->_entity)) {
      return '';
    }

    $count = $this->usageDataDatabaseStorage->fetchUsageData(
      $this->definition['usage_plugin_id'],
      $this->definition['usage_event_type'],
      $entity->getEntityTypeId(),
      $entity->id()
    );

    return $count === FALSE ? '' : (int) $count;
  }

}
